package com.ym.mec.biz.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ym.mec.biz.dal.dao.CourseHomeworkDao;
import com.ym.mec.biz.dal.dao.StudentLessonTrainingDetailMapper;
import com.ym.mec.biz.dal.dao.SubjectDao;
import com.ym.mec.biz.dal.dao.SysMusicScoreDao;
import com.ym.mec.biz.dal.dto.CourseHomeworkWrapper;
import com.ym.mec.biz.dal.dto.StudentLessonTrainingDetailWrapper;
import com.ym.mec.biz.dal.dto.StudentSubjectDto;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.ELessonTrainingType;
import com.ym.mec.biz.dal.enums.StandardEnum;
import com.ym.mec.biz.service.ExtracurricularExercisesService;
import com.ym.mec.biz.service.StudentLessonTrainingDetailService;
import com.ym.mec.biz.service.SubjectService;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 练习内容
 * 2023-03-31 14:18:47
 */
@Slf4j
@Service
public class StudentLessonTrainingDetailServiceImpl extends ServiceImpl<StudentLessonTrainingDetailMapper, StudentLessonTrainingDetail> implements StudentLessonTrainingDetailService {

    @Autowired
    private SysMusicScoreDao sysMusicScoreDao;

    @Autowired
    private SubjectDao subjectDao;

    @Autowired
    private CourseHomeworkDao courseHomeworkDao;

    @Autowired
    private SubjectService subjectService;

    @Autowired
    private ExtracurricularExercisesService extracurricularExercisesService;

    /**
     * 查询详情
     * @param id 详情ID
     * @return StudentLessonTrainingDetail
     */
    @Override
    public StudentLessonTrainingDetail detail(Long id) {

        return baseMapper.selectById(id);
    }

    /**
     * 添加
     * @param studentLessonTrainingDetail StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail
     * @return Boolean
     */
    @Override
    public Boolean add(StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail studentLessonTrainingDetail) {

        return this.save(JSON.parseObject(studentLessonTrainingDetail.jsonString(), StudentLessonTrainingDetail.class));
    }

    /**
     * 更新
     * @param studentLessonTrainingDetail StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail
     * @return Boolean
     */
    @Override
    public Boolean update(StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail studentLessonTrainingDetail){

        return this.updateById(JSON.parseObject(studentLessonTrainingDetail.jsonString(), StudentLessonTrainingDetail.class));
    }

    /**
     * 将作业标准 转换为 需要的作业详情
     *
     * @param detailList 作业标准
     * @param userIdList 学生ID
     * @param group
     */
    @Override
    public List<StudentLessonTrainingDetail> homeWorkDetail(List<StudentLessonTrainingDetailWrapper.AddStudentLessonTrainingDetail> detailList,
                                                            List<Integer> userIdList, Long homeworkId, ELessonTrainingType type, Integer group) {
        log.info("将作业标准 转换为 需要的作业详情 {},{}", detailList, userIdList);
        // 保存作业标准
        if (CollectionUtils.isEmpty(detailList) || CollectionUtils.isEmpty(userIdList)) {
            return new ArrayList<>();
        }
        if (group == null) {
            group = 1;
        }

        List<StudentLessonTrainingDetail> result = new ArrayList<>();

        List<StudentLessonTrainingDetail> details = JSON.parseArray(
            JSON.toJSONString(detailList), StudentLessonTrainingDetail.class);
        result.addAll(details);
        // 根据声部添加学生作业详情
        userIdList.forEach(userId -> {
            List<StudentLessonTrainingDetail> detailsUser = JSON.parseArray(
                JSON.toJSONString(detailList), StudentLessonTrainingDetail.class);
            for (StudentLessonTrainingDetail studentLessonTrainingDetail : detailsUser) {
                studentLessonTrainingDetail.setUserId(userId.longValue());
            }
            result.addAll(detailsUser);
        });
        for (StudentLessonTrainingDetail studentLessonTrainingDetail : result) {

            studentLessonTrainingDetail.setCourseHomeworkId(homeworkId);
            studentLessonTrainingDetail.setTrainingTimes(0);
            studentLessonTrainingDetail.setTrainingStatus(StandardEnum.NOT_START);
            studentLessonTrainingDetail.setCreateTime(new Date());
            studentLessonTrainingDetail.setUpdateTime(new Date());
            studentLessonTrainingDetail.setGroup(group);
            studentLessonTrainingDetail.setType(type);
        }
        return result;
    }

    /**
     * 查询布置的作业详情
     *
     * @param courseHomeworkId 作业ID
     * @param type             作业类型
     */
    @Override
    public List<StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail> getBaseLessonTrainingDetail(Long courseHomeworkId, ELessonTrainingType type) {
        return getBaseLessonTrainingDetail(courseHomeworkId, type, null);
    }

    /**
     * 学生端查询作业详情
     *
     * @param courseHomeworkId 作业ID
     * @param type             作业类型
     * @param userId           学生ID
     */
    @Override
    public List<StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail> getBaseLessonTrainingDetail(Long courseHomeworkId, ELessonTrainingType type, Long userId) {
        List<StudentLessonTrainingDetail> list = this.lambdaQuery()
                                                     .eq(StudentLessonTrainingDetail::getCourseHomeworkId,courseHomeworkId)
                                                     .isNull(userId == null,StudentLessonTrainingDetail::getUserId)
                                                     .eq(StudentLessonTrainingDetail::getType, type)
                                                     .eq(userId != null, StudentLessonTrainingDetail::getUserId, userId)
                                                     .list();
        if (CollectionUtils.isEmpty(list)) {
            return new ArrayList<>();
        }
        List<StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail> studentLessonTrainingDetails = JSON.parseArray(
            JSON.toJSONString(list), StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail.class);

         // 曲目id集合
        List<Long> collect = studentLessonTrainingDetails.stream()
                                                         .map(
                                                             StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail::getMusicScoreId)
                                                         .distinct()
                                                         .collect(Collectors.toList());

        List<SysMusicScore> scoreList = sysMusicScoreDao.findByIds(
            collect.stream().map(String::valueOf).collect(Collectors.joining(",")));

        // id 分组
        Map<Integer, SysMusicScore> musicScoreMap = scoreList.stream()
                                                     .collect(Collectors.toMap(SysMusicScore::getId, Function.identity()));

        for (StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail studentLessonTrainingDetail : studentLessonTrainingDetails) {
            SysMusicScore sysMusicScore = musicScoreMap.get(studentLessonTrainingDetail.getMusicScoreId().intValue());
            if (sysMusicScore != null) {
                studentLessonTrainingDetail.setMusicScoreName(sysMusicScore.getName());
            }
        }

        // 声部
        List<Integer> subjectIdList = studentLessonTrainingDetails.stream()
                                                          .map(o -> {
                                                              if (o.getSubjectId() != null) {
                                                                  return o.getSubjectId().intValue();
                                                              }
                                                              return 0;
                                                          })
                                                          .collect(Collectors.toList());

        List<Subject> subjectList = subjectDao.findBySubjectIds(subjectIdList);

        // id 分组
        Map<Integer, Subject> subjectMap = subjectList.stream()
                                                     .collect(Collectors.toMap(Subject::getId, Function.identity()));

        for (StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail studentLessonTrainingDetail : studentLessonTrainingDetails) {
            if (studentLessonTrainingDetail.getSubjectId() == null) {
                continue;
            }
            Subject subject = subjectMap.get(studentLessonTrainingDetail.getSubjectId().intValue());
            if (subject != null) {
                studentLessonTrainingDetail.setSubjectName(subject.getName());
            }
        }

        return studentLessonTrainingDetails;
    }


    /**
     * 查询作业详情练习组
     *
     * @param courseHomeworkId 作业ID
     * @param trainingType 作业类型
     */
    @Override
    public List<CourseHomeworkWrapper.StudentLessonTrainingStat> getLessonTrainingStat(Long courseHomeworkId, ELessonTrainingType trainingType) {
        List<StudentLessonTrainingDetail> list = this.lambdaQuery()
                                                     .eq(StudentLessonTrainingDetail::getCourseHomeworkId,courseHomeworkId)
                                                     .eq(StudentLessonTrainingDetail::getType, trainingType)
                                                    .isNotNull(StudentLessonTrainingDetail::getUserId)
                                                     .list();
        if (CollectionUtils.isEmpty(list)) {
            return new ArrayList<>();
        }

        // 用户ID集合
        List<Integer> userIdList = list.stream()
                                    .map(StudentLessonTrainingDetail::getUserId)
                                    .filter(Objects::nonNull)
                                    .distinct()
                                    .map(Long::intValue)
                                    .collect(Collectors.toList());
        String musicGroupId = null;
        if (trainingType.equals(ELessonTrainingType.HOMEWORK)) {
            CourseHomework courseHomework = courseHomeworkDao.get(courseHomeworkId);
            if (courseHomework == null) {
                return new ArrayList<>();
            }
            musicGroupId = courseHomework.getMusicGroupId();
        } else {
            ExtracurricularExercises exercises = extracurricularExercisesService.get(courseHomeworkId);
            if (exercises == null) {
                return new ArrayList<>();
            }
            musicGroupId = exercises.getMusicGroupId();
        }

        Map<Integer, StudentSubjectDto> studentSubjectDtoMap = getStudentMusicGroupSubject(userIdList, musicGroupId);

        // 根据group 分组
        Map<Integer, List<StudentLessonTrainingDetail>> groupMap = list.stream()
                                                                           .collect(Collectors.groupingBy(StudentLessonTrainingDetail::getGroup));


        Map<Integer, List<StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail>> groupDetailMap = getGroupMap(courseHomeworkId, trainingType);


        List<CourseHomeworkWrapper.StudentLessonTrainingStat> studentLessonTrainingStats = new ArrayList<>();

        // 根据分组数据统计声部， 完成人数
        groupMap.forEach((k,v) -> {
            CourseHomeworkWrapper.StudentLessonTrainingStat studentLessonTrainingStat = new CourseHomeworkWrapper.StudentLessonTrainingStat();
            studentLessonTrainingStat.setGroup(k);
            Set<Integer> userIds = v.stream().map(o -> o.getUserId().intValue()).collect(Collectors.toSet());
            // 课后作业和课外作业查询学生在当前乐团声部
            Set<String> subjectNameList = new HashSet<>();
            if(!CollectionUtils.isEmpty(userIds)) {
                for (Integer userId : userIds) {
                    StudentSubjectDto studentSubjectDto = studentSubjectDtoMap.get(userId);
                    if (studentSubjectDto != null) {
                        subjectNameList.add(studentSubjectDto.getSubjectName());
                    }
                }
            }
            // 按人分组
            Map<Long, List<StudentLessonTrainingDetail>> userMap = v.stream()
                                                                    .collect(Collectors.groupingBy(StudentLessonTrainingDetail::getUserId));
            int submitNum = 0;
            int passNum = 0;
            for (Map.Entry<Long, List<StudentLessonTrainingDetail>> entry : userMap.entrySet()) {
                List<StudentLessonTrainingDetail> v1 = entry.getValue();
                Set<StandardEnum> standardEnumSet = v1.stream().map(o -> o.getTrainingStatus()).filter(Objects::nonNull).collect(Collectors.toSet());
                if (standardEnumSet.size() > 1 || standardEnumSet.contains(StandardEnum.NOT_STANDARD)) {
                    submitNum++;
                }  else {
                    if (standardEnumSet.contains(StandardEnum.STANDARD)) {
                        submitNum++;
                        passNum++;
                    }
                }

            }

            studentLessonTrainingStat.setSubjectName(subjectNameList.stream().collect(Collectors.joining(",")));
            studentLessonTrainingStat.setStudentNum(userIds.size());
            studentLessonTrainingStat.setShouldSubmitNum(userIds.size());
            studentLessonTrainingStat.setSubmitNum(submitNum);
            studentLessonTrainingStat.setPassNum(passNum);

            //设置基本作业内容
            List<StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail> studentLessonTrainingDetails = groupDetailMap.get(k);
            if (!CollectionUtils.isEmpty(studentLessonTrainingDetails)) {
                studentLessonTrainingStat.setTrainingDetailList(studentLessonTrainingDetails);
            }
            studentLessonTrainingStats.add(studentLessonTrainingStat);

        });


        return studentLessonTrainingStats;
    }

    @NotNull
    @Override
    public Map<Integer, StudentSubjectDto> getStudentMusicGroupSubject(List<Integer> userIdList, String musicGroupId) {
        // 课后作业和课外作业查询学生在当前乐团声部
        Map<Integer, StudentSubjectDto> studentSubjectDtoMap = new HashMap<>();
        if (!StringUtils.isEmpty(musicGroupId)) {
            List<StudentRegistration> registerSubject = subjectDao.getRegisterSubject(musicGroupId, userIdList);
            if (!CollectionUtils.isEmpty(registerSubject)) {
                registerSubject.forEach(o -> {
                    StudentSubjectDto studentSubjectDto = new StudentSubjectDto();
                    studentSubjectDto.setSubjectId(o.getActualSubjectId());
                    studentSubjectDto.setSubjectName(o.getSubjectName());
                    studentSubjectDto.setStudentId(o.getUserId());
                    studentSubjectDtoMap.put(o.getUserId(), studentSubjectDto);
                });
            }
        }

        // 额外作业查询学生声部
        Map<Integer, StudentSubjectDto> subjectDtoMap = subjectService.getSubjectByStudentId(new HashSet(userIdList));
        subjectDtoMap.forEach((k,v) -> {
            if (!studentSubjectDtoMap.containsKey(k)) {
                studentSubjectDtoMap.put(k, v);
            }
        });
        return studentSubjectDtoMap;
    }

    @NotNull
    @Override
    public Map<Integer, List<StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail>> getGroupMap(Long courseHomeworkId, ELessonTrainingType trainingType) {
        // 作业内容
        List<StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail> baseLessonTrainingDetail = getBaseLessonTrainingDetail(
                courseHomeworkId, trainingType);

        // 按组分组
        Map<Integer, List<StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail>> groupDetailMap = baseLessonTrainingDetail.stream()
                                .collect(Collectors.groupingBy(StudentLessonTrainingDetailWrapper.StudentLessonTrainingDetail::getGroup,LinkedHashMap::new,Collectors.toList()));
        return groupDetailMap;
    }
}
